loadAnimalFood = {}

local modDirectory = g_currentModDirectory

function loadAnimalFood.loadMapData(self, superFunc, xmlFile)

    local returnValue = superFunc(self, xmlFile)

    local animalTypes = {}
    animalTypes["CHICKEN"] = AnimalType.CHICKEN
    animalTypes["COW"] = AnimalType.COW
    animalTypes["SHEEP"] = AnimalType.SHEEP
    animalTypes["PIG"] = AnimalType.PIG
    animalTypes["HORSE"] = AnimalType.HORSE

    local consumptionTypes = {}
    consumptionTypes["SERIAL"] = 1
    consumptionTypes["PARALLEL"] = 2

    if fileExists(modDirectory.."xml/animalFood.xml") then
        local foodXml = XMLFile.load("animalFood", modDirectory.."xml/animalFood.xml", AnimalFoodSystem.xmlSchema)

        --load food groups for animal pens/barns
        local i = 0
        local baseXmlPath = "animalFood.animals.animal("
        while true do
            local animalTypeName = getXMLString(foodXml.handle, baseXmlPath..i..")#animalType")
            local consumptionType = getXMLString(foodXml.handle, baseXmlPath..i..")#consumptionType")
            if animalTypeName ~= nil then
                local food = g_currentMission.animalFoodSystem:getAnimalFood(animalTypes[animalTypeName])
                if food ~= nil then
                    local j = 0
                    food.groups = {}
                    food.consumptionType = consumptionTypes[consumptionType]

                    local groupXmlPath = baseXmlPath..i..").foodGroup("

                    while true do
                        local title = getXMLString(foodXml.handle, groupXmlPath..j..")#title")
                        if title == nil then
                            break
                        end

                        local newFoodGroup = {}
                        newFoodGroup.title = g_i18n:getText(title) or title
                        newFoodGroup.productionWeight = getXMLFloat(foodXml.handle, groupXmlPath..j..")#productionWeight")
                        newFoodGroup.eatWeight = getXMLFloat(foodXml.handle, groupXmlPath..j..")#eatWeight") or 1
                        newFoodGroup.fillTypes = {}

                        for fill in string.gmatch( getXMLString(foodXml.handle, groupXmlPath..j..")#fillTypes"), "%S+") do
                            local ftIndex = g_fillTypeManager:getFillTypeIndexByName(fill) --This allows us to put fill types on that may not be in the map and it won't throw errors/warnings
                            if ftIndex ~= nil then
                                table.insert(newFoodGroup.fillTypes, ftIndex)
                            end
                        end
                        table.insert(food.groups, newFoodGroup)
                        j = j + 1
                    end
                end
            else
                break
            end
            i = i + 1
        end

        --load recipes
        self:loadRecipes(foodXml)

        --load optional ingredients
        local k = 0
        local recipePath = "animalFood.recipes.recipe("
        while true do
            local outputFillTypeName = getXMLString(foodXml.handle, recipePath..k..")#fillType")
            if outputFillTypeName == nil then
                break
            end
            local outputFillType = g_fillTypeManager:getFillTypeIndexByName(outputFillTypeName)
            if outputFillType ~= nil then
                local recipe = g_currentMission.animalFoodSystem:getRecipeByFillTypeIndex(outputFillType)
                if recipe ~= nil then
                    local m = 0
                    while true do
                        local optIngredFillTypeName = getXMLString(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#fillType")
                        if optIngredFillTypeName == nil then
                            break
                        end
                        local optIngredFillType = g_fillTypeManager:getFillTypeIndexByName(optIngredFillTypeName)
                        if optIngredFillType ~= nil then
                            local replaces = getXMLString(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#replaces")
                            if replaces == nil then
                                local newIngredient = {}
                                newIngredient.name = getXMLString(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#name")
                                newIngredient.title = getXMLString(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#title")
                                newIngredient.minPercentage = getXMLFloat(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#minPercentage") or 0
                                newIngredient.maxPercentage = getXMLFloat(foodXml.handle, recipePath..k..").optionalIngredient("..m..")#maxPercentage") or 100
                                newIngredient.fillTypes = {}
                                table.insert(newIngredient.fillTypes, optIngredFillType)
                                
                                table.insert(recipe.ingredients, newIngredient)
                            else
                                for _, ingredient in ipairs(recipe.ingredients) do
                                    for key, fillTypeIndex in ipairs(ingredient.fillTypes) do
                                        local fillType = g_fillTypeManager:getFillTypeNameByIndex(fillTypeIndex)
                                        if fillType == replaces then
                                            ingredient.fillTypes[key] = optIngredFillType
                                            break
                                        end
                                    end
                                end
                            end
                        end

                        m = m + 1
                    end
                end
            end

            k = k + 1
        end

        foodXml:delete()

    end

    return returnValue
end

AnimalFoodSystem.loadMapData = Utils.overwrittenFunction(AnimalFoodSystem.loadMapData, loadAnimalFood.loadMapData)